from argparse import ArgumentParser

from cc_core.commons.files import load_and_read, dump, file_extension, wrapped_print
from cc_core.commons.red import red_validation
from cc_core.commons.templates import inspect_templates_and_secrets, fill_template, fill_validation
from cc_core.commons.engines import engine_validation

from cc_faice.commons.red import dump_agent_cwl, dump_agent_job, dump_app_cwl
from cc_faice.commons.docker import dump_job


DESCRIPTION = 'Export RED_FILE to standard CWL compatible with cwltool.'


def attach_args(parser):
    parser.add_argument(
        'red_file', action='store', type=str, metavar='RED_FILE',
        help='RED_FILE (json or yaml) containing an experiment description as local path or http url.'
    )
    parser.add_argument(
        '--fill-file', action='store', type=str, metavar='FILL_FILE',
        help='FILL_FILE (json or yaml) containing key-value pairs for template variables in RED_FILE as '
             'local path or http url.'
    )
    parser.add_argument(
        '--outdir', action='store', type=str, metavar='OUTPUT_DIR',
        help='Output directory, default current directory. Will be passed to ccagent in the container.'
    )
    parser.add_argument(
        '--non-interactive', action='store_true',
        help='Do not ask for jinja template values interactively.'
    )
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )
    parser.add_argument(
        '--dump-prefix', action='store', type=str, metavar='DUMP_PREFIX', default='dumped_',
        help='Name prefix for files dumped to storage, default is "_dumped".'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    return run(**args.__dict__)


def run(red_file, fill_file, outdir, non_interactive, dump_format, dump_prefix):
    red_data = load_and_read(red_file, 'RED_FILE')
    red_validation(red_data, False, container_requirement=True)
    engine_validation(red_data, 'container', ['docker'], 'faice export')

    fill_data = None
    if fill_file:
        fill_data = load_and_read(fill_file, 'FILL_FILE')
        fill_validation(fill_data)

    template_keys_and_values, secret_values = inspect_templates_and_secrets(red_data, fill_data, non_interactive)
    red_data = fill_template(red_data, template_keys_and_values, False, True)

    if red_data['container']['settings']['image'].get('auth'):
        wrapped_print([
            'WARNING: cannot export container.settings.image.auth to cwl.',
            ''
        ], error=True)

    if 'batches' in red_data:
        wrapped_print([
            'ERROR: cannot export RED_FILE containing batches.',
            'Use "faice convert batches" to separate batches into individual files, then use "faice export" on the '
            'generated files.',
        ], error=True)
        return 1

    ext = file_extension(dump_format)
    dumped_app_cwl_file = '{}app-cli.cwl'.format(dump_prefix)
    dumped_app_red_file = '{}app-red.{}'.format(dump_prefix, ext)
    dumped_app_job_file = '{}app-job.{}'.format(dump_prefix, ext)
    dumped_agent_cwl_file = '{}agent-cli.cwl'.format(dump_prefix)
    dumped_agent_job_file = '{}agent-job.{}'.format(dump_prefix, ext)
    dumped_agent_job_ignore_output_file = '{}agent-job-ignore-output.{}'.format(dump_prefix, ext)
    agent_stdout_file = 'agent-stdout.{}'.format(ext)

    dumped_app_job_data = dump_job(red_data['inputs'], '.')
    dumped_agent_cwl_data = dump_agent_cwl(red_data, agent_stdout_file)
    dumped_agent_job_data = dump_agent_job(dumped_app_red_file, outdir, dump_format, False)
    dumped_agent_job_ignore_output_data = dump_agent_job(dumped_app_red_file, outdir, dump_format, True)
    dumped_app_cwl_data = dump_app_cwl(red_data)

    dump(dumped_app_cwl_data, dump_format, dumped_app_cwl_file)
    dump(red_data, dump_format, dumped_app_red_file)
    dump(dumped_app_job_data, dump_format, dumped_app_job_file)
    dump(dumped_agent_cwl_data, dump_format, dumped_agent_cwl_file)
    dump(dumped_agent_job_data, dump_format, dumped_agent_job_file)
    dump(dumped_agent_job_ignore_output_data, dump_format, dumped_agent_job_ignore_output_file)

    option3_command = 'cwltool {} {}'.format(dumped_app_cwl_file, dumped_app_job_file)
    if outdir:
        option3_command = '{} --outdir {}'.format(option3_command, outdir)

    wrapped_print([
        'OPTION 1:',
        'Use cwltool to execute app in container via "ccagent red" with support for REMOTE inputs and REMOTE outputs.',
        '$ cwltool {} {}'.format(dumped_agent_cwl_file, dumped_agent_job_file),
        '',
        'OPTION 2:',
        'Use cwltool to execute app in container via "ccagent red" with support for REMOTE inputs and LOCAL outputs.',
        '$ cwltool {} {}'.format(dumped_agent_cwl_file, dumped_agent_job_ignore_output_file),
        '',
        'OPTION 3:',
        'Use cwltool to execute app in container with support for LOCAL inputs and LOCAL outputs.',
        'ATTENTION: adjust input file paths in {}.'.format(dumped_app_job_file),
        '$ {}'.format(option3_command),
        '',
        'OPTION 4:',
        'Customize exported CWL and JOB files.'
    ])

    return 0



import sys
from argparse import ArgumentParser

from cc_core.commons.schema_map import schemas
from cc_core.commons.files import dump_print


DESCRIPTION = 'Write a jsonschema to stdout.'


def attach_args(parser):
    parser.add_argument(
        'schema', action='store', type=str, metavar='SCHEMA',
        help='SCHEMA as in "faice schemas list".'
    )
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    return run(**args.__dict__)


def run(schema, dump_format):
    if schema not in schemas:
        print('Schema "{}" not found. Use "faice schema list" for available schemas.'.format(schema), file=sys.stderr)
        return 1

    dump_print(schemas[schema], dump_format)
    return 0



from collections import OrderedDict

from cc_faice.agent.cwl.main import main as cwl_main
from cc_faice.agent.red.main import main as red_main

from cc_faice.agent.cwl.main import DESCRIPTION as CWL_DESCRIPTION
from cc_faice.agent.red.main import DESCRIPTION as RED_DESCRIPTION

from cc_core.commons.cli_modes import cli_modes


SCRIPT_NAME = 'faice agent'
TITLE = 'modes'
DESCRIPTION = 'Run a RED experiment or a CWL CommandLineTool.'
MODES = OrderedDict([
    ('cwl', {'main': cwl_main, 'description': CWL_DESCRIPTION}),
    ('red', {'main': red_main, 'description': RED_DESCRIPTION})
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES)



import os
from uuid import uuid4
from argparse import ArgumentParser

from cc_core.commons.files import load_and_read, dump, dump_print, file_extension
from cc_core.commons.exceptions import exception_format, RedValidationError
from cc_core.commons.red import red_validation
from cc_core.commons.templates import fill_validation, fill_template, inspect_templates_and_secrets
from cc_core.commons.engines import engine_validation

from cc_faice.commons.docker import DockerManager, docker_result_check


DESCRIPTION = 'Run an experiment as described in a RED_FILE in a container with ccagent (cc_core.agent.cwl_io).'


def attach_args(parser):
    parser.add_argument(
        'red_file', action='store', type=str, metavar='RED_FILE',
        help='RED_FILE (json or yaml) containing an experiment description as local path or http url.'
    )
    parser.add_argument(
        '--fill-file', action='store', type=str, metavar='FILL_FILE',
        help='FILL_FILE (json or yaml) containing key-value pairs for template variables in RED_FILE as '
             'local path or http url.'
    )
    parser.add_argument(
        '--outdir', action='store', type=str, metavar='OUTPUT_DIR',
        help='Output directory, default current directory. Will be passed to ccagent in the container.'
    )
    parser.add_argument(
        '--disable-pull', action='store_true',
        help='Do not try to pull Docker images.'
    )
    parser.add_argument(
        '--leave-container', action='store_true',
        help='Do not delete Docker container used by jobs after they exit.'
    )
    parser.add_argument(
        '--non-interactive', action='store_true',
        help='Do not ask for jinja template values interactively.'
    )
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )
    parser.add_argument(
        '--dump-prefix', action='store', type=str, metavar='DUMP_PREFIX', default='dumped_',
        help='Name prefix for files dumped to storage, default is "dumped_".'
    )
    parser.add_argument(
        '--ignore-outputs', action='store_true',
        help='Ignore RED connectors specified in RED_FILE outputs section.'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()

    result = run(**args.__dict__)
    dump_print(result, args.dump_format)

    if result['state'] == 'succeeded':
        return 0

    return 1


def run(
        red_file,
        fill_file,
        outdir,
        disable_pull,
        leave_container,
        non_interactive,
        dump_format,
        dump_prefix,
        ignore_outputs
):
    result = {
        'containers': [],
        'debugInfo': None,
        'state': 'succeeded'
    }

    secret_values = None
    ext = file_extension(dump_format)
    dumped_fill_file = '{}fill.{}'.format(dump_prefix, ext)

    try:
        red_data = load_and_read(red_file, 'RED_FILE')
        red_validation(red_data, ignore_outputs, container_requirement=True)
        engine_validation(red_data, 'container', ['docker'], 'faice agent red')

        fill_data = None
        if fill_file:
            fill_data = load_and_read(fill_file, 'FILL_FILE')
            fill_validation(fill_data)

        template_keys_and_values, secret_values = inspect_templates_and_secrets(red_data, fill_data, non_interactive)

        if template_keys_and_values:
            dump(template_keys_and_values, dump_format, dumped_fill_file)

        ram = red_data['container']['settings'].get('ram')

        docker_manager = DockerManager()

        image = red_data['container']['settings']['image']['url']
        registry_auth = red_data['container']['settings']['image'].get('auth')
        registry_auth = fill_template(registry_auth, template_keys_and_values, True, True)

        if not disable_pull:
            docker_manager.pull(image, auth=registry_auth)

    except RedValidationError:
        result['debugInfo'] = exception_format(secret_values=secret_values)
        result['state'] = 'failed'
        return result
    except:
        result['debugInfo'] = exception_format()
        result['state'] = 'failed'
        return result

    batches = [None]
    if 'batches' in red_data:
        batches = list(range(len(red_data['batches'])))

    for batch in batches:
        container_result = {
            'command': None,
            'name': None,
            'volumes': {
                'readOnly': None,
                'readWrite': None
            },
            'ccagent': None,
            'debugInfo': None,
            'state': 'succeeded'
        }
        result['containers'].append(container_result)
        try:
            if batch is None:
                work_dir = 'work'
            else:
                work_dir = 'work_{}'.format(batch)

            mapped_work_dir = '/opt/cc/work'
            mapped_red_file = '/opt/cc/red.{}'.format(ext)
            mapped_fill_file = '/opt/cc/fill.{}'.format(ext)

            container_name = str(uuid4())
            container_result['name'] = container_name

            command = [
                'ccagent',
                'red',
                mapped_red_file,
                '--dump-format={}'.format(dump_format)
            ]

            if batch is not None:
                command.append('--batch={}'.format(batch))

            if outdir:
                command.append('--outdir={}'.format(outdir))

            if ignore_outputs:
                command.append('--ignore-outputs')

            if template_keys_and_values:
                command.append('--fill-file={}'.format(mapped_fill_file))

            command = ' '.join([str(c) for c in command])

            container_result['command'] = command

            ro_mappings = [[os.path.abspath(red_file), mapped_red_file]]
            rw_mappings = [[os.path.abspath(work_dir), mapped_work_dir]]

            if template_keys_and_values:
                rw_mappings.append([os.path.abspath(dumped_fill_file), mapped_fill_file])

            container_result['volumes']['readOnly'] = ro_mappings
            container_result['volumes']['readWrite'] = rw_mappings

            if not os.path.exists(work_dir):
                os.makedirs(work_dir)

            ccagent_data = docker_manager.run_container(
                container_name, image, command, ro_mappings, rw_mappings, mapped_work_dir, leave_container, ram
            )
            container_result['ccagent'] = ccagent_data
            docker_result_check(ccagent_data)
        except:
            container_result['debugInfo'] = exception_format()
            container_result['state'] = 'failed'
            result['state'] = 'failed'
            break

    if os.path.exists(dumped_fill_file):
        os.remove(dumped_fill_file)

    return result



import os
from argparse import ArgumentParser
from flask import Flask, send_from_directory, request
from werkzeug.utils import secure_filename


DESCRIPTION = 'Run a simple http file server for debugging purposes. USE CAREFULLY: this server does not enable ' \
              'encryption and does not verify authorization credentials.'


def attach_args(parser):
    parser.add_argument(
        '--bind-host', action='store', type=str, metavar='BIND_HOST', default='0.0.0.0',
        help='Bind server to a network interface like "172.17.0.1" (docker) or "127.0.0.1" (localhost). '
             'Default is "0.0.0.0" (all interfaces).'
    )
    parser.add_argument(
        '--bind-port', action='store', type=int, metavar='BIND_PORT', default=5000,
        help='Bind server to a tcp port. Default is 5000.'
    )
    parser.add_argument(
        '-u', '--updir', action='store', type=str, metavar='UPLOAD_DIR', default=os.getcwd(),
        help='Specify a directory for file uploads. Default is current working directory.'
    )
    parser.add_argument(
        '-d', '--downdir', action='store', type=str, metavar='DOWNLOAD_DIR', default=os.getcwd(),
        help='Specify a directory for file downloads. Default is current working directory.'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    return run(**args.__dict__)


def run(bind_host, bind_port, updir, downdir):
    app = Flask('faice file-server')

    @app.route('/<file_name>', methods=['GET'])
    def get_file(file_name):
        """
        .. :quickref: API; Download file
        Download a file specified as <file_name> from DOWNLOAD_DIR.

        .. sourcecode:: http
            GET /<file_name> HTTP/1.1

        """
        file_name = secure_filename(file_name)
        return send_from_directory(os.path.expanduser(downdir), file_name, as_attachment=True)

    @app.route('/<file_name>', methods=['POST'])
    def post_file(file_name):
        """
        .. :quickref: API; Upload file
        Upload a file specified as <file_name> to UPLOAD_DIR.

        .. sourcecode:: http
            POST /<file_name> HTTP/1.1

        """
        file_name = secure_filename(file_name)
        file_path = os.path.join(os.path.expanduser(updir), file_name)
        with open(file_path, 'wb') as f:
            f.write(request.data)
        return ''

    app.run(host=bind_host, port=bind_port)



import sys
import jsonschema
from argparse import ArgumentParser

from cc_core.commons.schema_map import schemas
from cc_core.commons.files import load_and_read


DESCRIPTION = 'Validate data against schema. Returns code 0 if data is valid.'


def attach_args(parser):
    parser.add_argument(
        'schema', action='store', type=str, metavar='SCHEMA',
        help='SCHEMA as in "faice schemas list".'
    )
    parser.add_argument(
        'data_file', action='store', type=str, metavar='DATA_FILE',
        help='DATA_FILE (json or yaml) to be validated as local path or http url.'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    return run(**args.__dict__)


def run(schema, data_file):
    if schema not in schemas:
        print('Schema "{}" not found. Use "faice schema list" for available schemas.'.format(schema), file=sys.stderr)
        return 1

    data = load_and_read(data_file, 'DATA_FILE')
    jsonschema.validate(data, schemas[schema])

    return 0



import os
from uuid import uuid4
from argparse import ArgumentParser
from pprint import pprint

from cc_core.commons.files import load_and_read, dump, dump_print, file_extension
from cc_core.commons.cwl import cwl_validation
from cc_core.commons.exceptions import exception_format

from cc_faice.commons.docker import dump_job, input_volume_mappings, DockerManager, docker_result_check


DESCRIPTION = 'Run a CommandLineTool as described in a CWL_FILE and its corresponding JOB_FILE in a container with ' \
              'ccagent (cc_core.agent.cwl).'


def attach_args(parser):
    parser.add_argument(
        'cwl_file', action='store', type=str, metavar='CWL_FILE',
        help='CWL_FILE (json or yaml) containing a CLI description as local path or http url.'
    )
    parser.add_argument(
        'job_file', action='store', type=str, metavar='JOB_FILE',
        help='JOB_FILE (json or yaml) in the CWL job format as local path or http url.'
    )
    parser.add_argument(
        '--outdir', action='store', type=str, metavar='OUTPUT_DIR',
        help='Output directory, default current directory. Will be passed to ccagent in the container.'
    )
    parser.add_argument(
        '--disable-pull', action='store_true',
        help='Do not try to pull Docker images.'
    )
    parser.add_argument(
        '--leave-container', action='store_true',
        help='Do not delete Docker container used by jobs after they exit.'
    )
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )
    parser.add_argument(
        '--dump-prefix', action='store', type=str, metavar='DUMP_PREFIX', default='dumped_',
        help='Name prefix for files dumped to storage, default is "dumped_".'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()

    result = run(**args.__dict__)
    dump_print(result, args.dump_format)

    if result['state'] == 'succeeded':
        return 0

    return 1


def run(cwl_file, job_file, outdir, disable_pull, leave_container, dump_format, dump_prefix):
    result = {
        'container': {
            'command': None,
            'name': None,
            'volumes': {
                'readOnly': None,
                'readWrite': None
            },
            'ccagent': None
        },
        'debugInfo': None,
        'state': 'succeeded'
    }

    try:
        cwl_data = load_and_read(cwl_file, 'CWL_FILE')
        job_data = load_and_read(job_file, 'JOB_FILE')

        cwl_validation(cwl_data, job_data, docker_requirement=True)

        ext = file_extension(dump_format)
        input_dir = os.path.split(os.path.expanduser(job_file))[0]
        work_dir = 'work'
        dumped_job_file = '{}job.{}'.format(dump_prefix, ext)

        mapped_input_dir = '/opt/cc/inputs'
        mapped_work_dir = '/opt/cc/work'
        mapped_cwl_file = '/opt/cc/cli.cwl'
        mapped_job_file = '/opt/cc/job.{}'.format(ext)

        dumped_job_data = dump_job(job_data, mapped_input_dir)

        ro_mappings = input_volume_mappings(job_data, dumped_job_data, input_dir)
        ro_mappings += [
            [os.path.abspath(cwl_file), mapped_cwl_file],
            [os.path.abspath(dumped_job_file), mapped_job_file]
        ]
        rw_mappings = [[os.path.abspath(work_dir), mapped_work_dir]]

        result['container']['volumes']['readOnly'] = ro_mappings
        result['container']['volumes']['readWrite'] = rw_mappings

        container_name = str(uuid4())
        result['container']['name'] = container_name
        docker_manager = DockerManager()

        image = cwl_data['requirements']['DockerRequirement']['dockerPull']
        if not disable_pull:
            docker_manager.pull(image)

        command = [
            'ccagent',
            'cwl',
            mapped_cwl_file,
            mapped_job_file,
            '--dump-format={}'.format(dump_format)
        ]

        if outdir:
            command += [
                '--outdir={}'.format(outdir)
            ]

        command = ' '.join([str(c) for c in command])

        result['container']['command'] = command

        if not os.path.exists(work_dir):
            os.makedirs(work_dir)

        dump(dumped_job_data, dump_format, dumped_job_file)

        ccagent_data = docker_manager.run_container(
            container_name, image, command, ro_mappings, rw_mappings, mapped_work_dir, leave_container, None
        )
        result['container']['ccagent'] = ccagent_data
        docker_result_check(ccagent_data)
    except:
        result['debugInfo'] = exception_format()
        result['state'] = 'failed'

    return result



from argparse import ArgumentParser

from cc_core.commons.schema_map import schemas
from cc_core.commons.files import dump_print


DESCRIPTION = 'List of all available jsonschemas defined in cc-core.'


def attach_args(parser):
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    run(**args.__dict__)
    return 0


def run(dump_format):
    dump_print({'schemas': list(schemas.keys())}, dump_format)



from collections import OrderedDict

from cc_faice.convert.batches.main import main as batches_main
from cc_faice.convert.format.main import main as format_main

from cc_faice.convert.batches.main import DESCRIPTION as BATCHES_DESCRIPTION
from cc_faice.convert.format.main import DESCRIPTION as FORMAT_DESCRIPTION

from cc_core.commons.cli_modes import cli_modes


SCRIPT_NAME = 'faice convert'
TITLE = 'modes'
DESCRIPTION = 'File conversion utilities.'
MODES = OrderedDict([
    ('batches', {'main': batches_main, 'description': BATCHES_DESCRIPTION}),
    ('format', {'main': format_main, 'description': FORMAT_DESCRIPTION}),
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES)


from collections import OrderedDict

from cc_faice.version import VERSION
from cc_faice.commons.compatibility import version_validation

from cc_faice.agent.main import main as agent_main
from cc_faice.schema.main import main as schema_main
from cc_faice.file_server.main import main as file_server_main
from cc_faice.export.main import main as export_main
from cc_faice.convert.main import main as convert_main

from cc_faice.agent.main import DESCRIPTION as AGENT_DESCRIPTION
from cc_faice.schema.main import DESCRIPTION as SCHEMA_DESCRIPTION
from cc_faice.file_server.main import DESCRIPTION as FILE_SERVER_DESCRIPTION
from cc_faice.export.main import DESCRIPTION as EXPORT_DESCRIPTION
from cc_faice.convert.main import DESCRIPTION as CONVERT_DESCRIPTION

from cc_core.commons.cli_modes import cli_modes


SCRIPT_NAME = 'faice'
TITLE = 'tools'
DESCRIPTION = 'FAICE Copyright (C) 2018  Christoph Jansen. This software is distributed under the AGPL-3.0 ' \
              'LICENSE and is part of the Curious Containers project (https://www.curious-containers.cc).'
MODES = OrderedDict([
    ('agent', {'main': agent_main, 'description': AGENT_DESCRIPTION}),
    ('schema', {'main': schema_main, 'description': SCHEMA_DESCRIPTION}),
    ('convert', {'main': convert_main, 'description': CONVERT_DESCRIPTION}),
    ('export', {'main': export_main, 'description': EXPORT_DESCRIPTION}),
    ('file-server', {'main': file_server_main, 'description': FILE_SERVER_DESCRIPTION})
])


def main():
    version_validation()
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES, VERSION)



from argparse import ArgumentParser

from cc_core.commons.files import load_and_read, file_extension, wrapped_print, dump
from cc_core.commons.red import red_validation, convert_batch_experiment


DESCRIPTION = 'Convert batches from a single RED_FILE into separate files containing only one batch each.'


def attach_args(parser):
    parser.add_argument(
        'red_file', action='store', type=str, metavar='RED_FILE',
        help='RED_FILE (json or yaml) containing an experiment description as local path or http url.'
    )
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )
    parser.add_argument(
        '--dump-prefix', action='store', type=str, metavar='DUMP_PREFIX', default='dumped_',
        help='Name prefix for files dumped to storage, default is "dumped_".'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    return run(**args.__dict__)


def run(red_file, dump_format, dump_prefix):
    ext = file_extension(dump_format)

    red_data = load_and_read(red_file, 'RED_FILE')
    red_validation(red_data, False)

    if 'batches' not in red_data:
        wrapped_print([
            'ERROR: RED_FILE does not contain batches.'
        ], error=True)
        return 1

    for batch in range(len(red_data['batches'])):
        batch_data = convert_batch_experiment(red_data, batch)
        dumped_batch_file = '{}batch_{}.{}'.format(dump_prefix, batch, ext)
        dump(batch_data, dump_format, dumped_batch_file)

    return 0



from argparse import ArgumentParser

from cc_core.commons.files import dump_print, load_and_read


DESCRIPTION = 'Read an arbitrary JSON or YAML file and convert it into the specified format.'


def attach_args(parser):
    parser.add_argument(
        'file', action='store', type=str, metavar='FILE',
        help='FILE (json or yaml) to be converted into specified DUMP_FORMAT as local path or http url.'
    )
    parser.add_argument(
        '--dump-format', action='store', type=str, metavar='DUMP_FORMAT', choices=['json', 'yaml', 'yml'],
        default='yaml', help='Dump format for data written to files or stdout, choices are "json" or "yaml", default '
                             'is "yaml".'
    )


def main():
    parser = ArgumentParser(description=DESCRIPTION)
    attach_args(parser)
    args = parser.parse_args()
    run(**args.__dict__)
    return 0


def run(file, dump_format):
    data = load_and_read(file, 'FILE')
    dump_print(data, dump_format)



import os
import docker

from cc_core.commons.cwl import location
from cc_core.commons.files import read
from cc_core.commons.exceptions import AgentError


def docker_result_check(ccagent_data):
    if ccagent_data['state'] != 'succeeded':
        raise AgentError('ccagent did not succeed')


def dump_job(job_data, mapped_input_dir):
    job = {}

    for key, arg in job_data.items():
        val = arg
        if isinstance(arg, list):
            val = []
            for index, i in enumerate(arg):
                if isinstance(i, dict):
                    path = os.path.join(mapped_input_dir, '{}_{}'.format(key, index))
                    val.append({
                        'class': 'File',
                        'path': path
                    })
                else:
                    val.append(i)
        elif isinstance(arg, dict):
            path = os.path.join(mapped_input_dir, key)
            val = {
                'class': 'File',
                'path': path
            }

        job[key] = val

    return job


def input_volume_mappings(job_data, dumped_job_data, input_dir):
    volumes = []

    for key, val in job_data.items():
        if isinstance(val, list) and len(val) > 0 and isinstance(val[0], dict):
            for i, e in enumerate(val):
                file_path = location(key, e)

                if not os.path.isabs(file_path):
                    file_path = os.path.join(os.path.expanduser(input_dir), file_path)

                container_file_path = dumped_job_data[key][i]['path']
                volumes.append([os.path.abspath(file_path), container_file_path])

        if isinstance(val, dict):
            file_path = location(key, val)

            if not os.path.isabs(file_path):
                file_path = os.path.join(os.path.expanduser(input_dir), file_path)

            container_file_path = dumped_job_data[key]['path']
            volumes.append([os.path.abspath(file_path), container_file_path])

    return volumes


class DockerManager:
    def __init__(self):
        self._client = docker.DockerClient(
            base_url='unix://var/run/docker.sock',
            version='auto'
        )

    def pull(self, image, auth=None):
        self._client.images.pull(image, auth_config=auth)

    def run_container(self, name, image, command, ro_mappings, rw_mappings, work_dir, leave_container, ram):
        binds = {}

        for host_vol, container_vol in ro_mappings:
            binds[host_vol] = {
                'bind': container_vol,
                'mode': 'ro'
            }

        for host_vol, container_vol in rw_mappings:
            binds[host_vol] = {
                'bind': container_vol,
                'mode': 'rw'
            }

        mem_limit = None

        if ram is not None:
            mem_limit = '{}m'.format(ram)

        c = self._client.containers.create(
            image,
            command,
            volumes=binds,
            name=name,
            user='1000:1000',
            working_dir=work_dir,
            mem_limit=mem_limit,
            memswap_limit=mem_limit
        )

        c.start()
        c.wait()
        std_out = c.logs()

        if not leave_container:
            c.remove()

        print(std_out.decode('utf-8'))

        return read(std_out.decode('utf-8'), 'CCAGENT_OUTPUT')



from collections import OrderedDict

from cc_faice.schema.list.main import main as list_main
from cc_faice.schema.show.main import main as show_main
from cc_faice.schema.validate.main import main as validate_main

from cc_faice.schema.list.main import DESCRIPTION as LIST_DESCRIPTION
from cc_faice.schema.show.main import DESCRIPTION as SHOW_DESCRIPTION
from cc_faice.schema.validate.main import DESCRIPTION as VALIDATE_DESCRIPTION

from cc_core.commons.cli_modes import cli_modes


SCRIPT_NAME = 'faice schema'
TITLE = 'modes'
DESCRIPTION = 'List or show jsonschemas defined in cc-core.'
MODES = OrderedDict([
    ('list', {'main': list_main, 'description': LIST_DESCRIPTION}),
    ('show', {'main': show_main, 'description': SHOW_DESCRIPTION}),
    ('validate', {'main': validate_main, 'description': VALIDATE_DESCRIPTION})
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES)



import cc_core.version
import cc_faice.version
from cc_core.commons.files import wrapped_print


def version_validation():
    compatible = True
    if not cc_core.version.RED_VERSION == cc_faice.version.RED_VERSION:
        compatible = False
    if not cc_core.version.CC_VERSION == cc_faice.version.CC_VERSION:
        compatible = False

    if not compatible:
        wrapped_print([
            'WARNING: cc-faice {0}.{1}.{2} is not compatible with cc-core {3}.{4}.{5}. '
            'Install cc-core {0}.{1}.x or cc-faice {3}.{4}.y'.format(
                cc_faice.version.RED_VERSION, cc_faice.version.CC_VERSION, cc_faice.version.PACKAGE_VERSION,
                cc_core.version.RED_VERSION, cc_core.version.CC_VERSION, cc_core.version.PACKAGE_VERSION
            )
        ], error=True)



from copy import deepcopy


def dump_agent_cwl(red_data, stdout_file):
    outputs = deepcopy(red_data['cli']['outputs'])
    outputs['standard-out'] = {
        'type': 'stdout'
    }

    return {
        'cwlVersion': 'v1.0',
        'class': 'CommandLineTool',
        'baseCommand': ['ccagent', 'red'],
        'doc': '',
        'requirements': {
            'DockerRequirement': {
                'dockerPull': red_data['container']['settings']['image']['url']
            }
        },
        'inputs': {
            'red-file': {
                'type': 'File',
                'inputBinding': {
                    'position': 0
                }
            },
            'outdir': {
                'type': 'string?',
                'inputBinding': {
                    'prefix': '--outdir=',
                    'separate': False
                }
            },
            'dump-format': {
                'type': 'string?',
                'inputBinding': {
                    'prefix': '--dump-format=',
                    'separate': False
                }
            },
            'ignore_outputs': {
                'type': 'boolean?',
                'inputBinding': {
                    'prefix': '--ignore-outputs',
                }
            },
            'return_zero': {
                'type': 'boolean?',
                'inputBinding': {
                    'prefix': '--return-zero',
                }
            }
        },
        'outputs': outputs,
        'stdout': stdout_file
    }


def dump_agent_job(app_red_file, outdir, dump_format, ignore_outputs):
    agent_job = {
        'red-file': {
            'class': 'File',
            'path': app_red_file
        },
        'dump_format': dump_format
    }

    if outdir:
        agent_job['outdir'] = outdir

    if ignore_outputs:
        agent_job['ignore_outputs'] = ignore_outputs

    return agent_job


def dump_app_cwl(red_data):
    app_cwl = deepcopy(red_data['cli'])
    app_cwl['requirements'] = {
        'DockerRequirement': {
            'dockerPull': red_data['container']['settings']['image']['url']
        }
    }
    return app_cwl
